package com.weaver.secondev.action.approval;

import com.alibaba.fastjson.JSONObject;
import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.enums.SourceType;
import com.weaver.esb.api.rpc.EsbServerlessRpcRemoteInterface;
import com.weaver.secondev.util.DataOperateUtil;
import com.weaver.secondev.util.ThirdInfoUtil;
import com.weaver.verupgrade.interfaces.workflow.action.Action;
import lombok.extern.slf4j.Slf4j;
import com.weaver.verupgrade.soa.workflow.request.RequestInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: calyrex
 * @CreateTime: 2025-06-03
 * @Description: 应付单审批推送
 */

@Slf4j
@Service("workflow_action_ReimbursementApprovalAction")
public class ReimbursementApprovalAction implements EsbServerlessRpcRemoteInterface {

    @Autowired
    private RequestInfo requestInfoTemp;

    @Autowired
    private DataOperateUtil dataOperateUtil;

    @Autowired
    private ThirdInfoUtil thirdInfoUtil;

    @Override
    public WeaResult<Map<String, Object>> execute(Map<String, Object> params) {
        log.error("ReimbursementApprovalAction execute start");
        log.error("ReimbursementApprovalAction execute params : " + params);
        try {
            Map<String, Object> weaverResultMap = new HashMap<>();
            Long  requestIdTemp = Long.parseLong(String.valueOf(params.getOrDefault("requestid", -1)));
            Long  userIdTemp = Long.parseLong(String.valueOf(params.getOrDefault("userid", -1)));
            String status = String.valueOf(params.getOrDefault("status", ""));
            RequestInfo requestinfo = requestInfoTemp.getRequestInfo(requestIdTemp, userIdTemp);
            String tableName = requestinfo.getRequestManager().getBillTableName();
            String formDataId = dataOperateUtil.getFormDataId(String.valueOf(requestIdTemp));
            String sql = "select pk,billtype from " + tableName + " where form_data_id = " + formDataId;
            String sourceType = String.valueOf(SourceType.LOGIC);
            String groupId = "weaver-workflow-report-serviceworkflowreport";
            Map<String, Object> datas = dataOperateUtil.executeForQuery(sourceType, groupId, sql);
            if(String.valueOf(datas.get("status")).equals("OK")){
                List<Map<String,Object>> records = (List<Map<String,Object>>)datas.get("records");
                if (records.size()>0){
                    Map<String, Object> map = records.get(0);
                    String pk = String.valueOf(map.get("pk"));
                    String billtype = String.valueOf(map.get("billtype"));
                    String usercode = dataOperateUtil.getConfig("third_usercode");
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("pk", pk);
                    jsonObject.put("usercode", usercode);
                    jsonObject.put("billtype", billtype);
                    String url = "";
                    if ("1".equals(status)){
                        url = dataOperateUtil.getConfig("ip_address") + "/nccloud/api/arap/payablebill/commit";
                    }else {
                        url = dataOperateUtil.getConfig("ip_address") + "/nccloud/api/arap/payablebill/uncommit";
                    }
                    JSONObject tokenInfo = thirdInfoUtil.getToken();
                    if (tokenInfo.get("success").equals("true")){
                        JSONObject data = tokenInfo.getJSONObject("data");
                        String token = data.getString("access_token");
                        Boolean result = thirdInfoUtil.transferStatusInfo(url, token, jsonObject);
                        log.info("ReimbursementApprovalAction execute result : " + result);
                        if (result){
                            return WeaResult.success(thirdInfoUtil.getResultMapForAction(weaverResultMap, "result", Action.SUCCESS, requestinfo.getRequestManager()));
                        }else {
                            log.info("ReimbursementApprovalAction execute error--推送失败");
                            return WeaResult.fail("ReimbursementApprovalAction execute error--推送失败");
                        }
                    }else {
                        log.info("ReimbursementApprovalAction execute error--token调用失败");
                        return WeaResult.fail("ReimbursementApprovalAction execute error--token调用失败");
                    }
                }else {
                    log.info("ReimbursementApprovalAction execute error--查询不到数据");
                    return WeaResult.fail("ReimbursementApprovalAction execute error--查询不到数据");
                }
            }else {
                log.info("ReimbursementApprovalAction execute error--查询失败");
                return WeaResult.fail("ReimbursementApprovalAction execute error--查询失败");
            }
        } catch (Exception e) {
            log.error("ReimbursementApprovalAction execute error", e);
            return WeaResult.fail("ReimbursementApprovalAction execute error", e);
        }
    }
}
