package com.weaver.secondev.action.info;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.enums.SourceType;
import com.weaver.esb.api.rpc.EsbServerlessRpcRemoteInterface;
import com.weaver.file.ud.api.FileDownloadService;
import com.weaver.secondev.util.DataOperateUtil;
import com.weaver.secondev.util.ThirdInfoUtil;
import com.weaver.verupgrade.interfaces.workflow.action.Action;
import com.weaver.verupgrade.soa.workflow.request.RequestInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: calyrex
 * @CreateTime: 2025-06-11
 * @Description: 请购单数据回写
 */


@Slf4j
@Service("workflow_action_PurchaseRequisitionInfoAction")
public class PurchaseRequisitionInfoAction implements EsbServerlessRpcRemoteInterface {

    @Autowired
    private RequestInfo requestInfoTemp;

    @Autowired
    private DataOperateUtil dataOperateUtil;

    @Autowired
    private ThirdInfoUtil thirdInfoUtil;

    @Autowired
    private FileDownloadService fileDownloadService;

    @Override
    public WeaResult<Map<String, Object>> execute(Map<String, Object> params) {
        log.info("PurchaseRequisitionInfoAction execute start");
        log.info("PurchaseRequisitionInfoAction execute params : " + params);
        try {
            Map<String, Object> weaverResultMap = new HashMap<>();
            Long  requestIdTemp = Long.parseLong(String.valueOf(params.getOrDefault("requestid", -1)));
            Long  userIdTemp = Long.parseLong(String.valueOf(params.getOrDefault("userid", -1)));
            RequestInfo requestinfo = requestInfoTemp.getRequestInfo(requestIdTemp, userIdTemp);
            String tableName = requestinfo.getRequestManager().getBillTableName();
            String wokflowId = requestinfo.getWorkflowid();
            String formDataId = dataOperateUtil.getFormDataId(String.valueOf(requestIdTemp));
            String sql = "select * from " + tableName + " where formdataid = " + formDataId;
            log.error("ReimbursementInfoAction execute sql : " + sql);
            String sourceType = String.valueOf(SourceType.LOGIC);
            String groupId = "weaver-workflow-report-serviceworkflowreport";
            Map<String, Object> datas = dataOperateUtil.executeForQuery(sourceType, groupId, sql);
            JSONObject responseJson = new JSONObject();
            if(String.valueOf(datas.get("status")).equals("OK")){
                List<Map<String,Object>> records = (List<Map<String,Object>>)datas.get("records");
                if (records.size()>0){
                    JSONObject po_praybill = new JSONObject();
                    String ccurrencyid = "CNY";
                    String pk_org = dataOperateUtil.null2String(records.get(0).get("pk_org"));
                    String pk_org_v = dataOperateUtil.null2String(records.get(0).get("pk_org_v"));
                    String dbilldate = dataOperateUtil.null2String(records.get(0).get("dbilldate"));
                    String ctrantypeid = "D1";
                    String pk_planpsn = dataOperateUtil.getJobNum(dataOperateUtil.null2String(records.get(0).get("pk_planpsn"))).toString();
                    String pk_plandept = dataOperateUtil.getDepartmentCode(dataOperateUtil.null2String(records.get(0).get("pk_plandept"))).toString();
                    String fbillstatus = dataOperateUtil.null2String(records.get(0).get("fbillstatus"));
                    String vmemo = dataOperateUtil.null2String(records.get(0).get("vmemo"));
                    String billmaker = dataOperateUtil.getJobNum(dataOperateUtil.null2String(records.get(0).get("billmaker"))).toString();
                    String dmakedate = dataOperateUtil.null2String(records.get(0).get("dmakedate"));
                    po_praybill.put("ccurrencyid", ccurrencyid);
                    po_praybill.put("pk_org", pk_org);
                    po_praybill.put("pk_org_v", pk_org_v);
                    po_praybill.put("dbilldate", dbilldate);
                    po_praybill.put("ctrantypeid", ctrantypeid);
                    po_praybill.put("pk_planpsn", pk_planpsn);
                    po_praybill.put("pk_plandept", pk_plandept);
                    po_praybill.put("fbillstatus", fbillstatus);
                    po_praybill.put("vmemo", vmemo);
                    po_praybill.put("billmaker", billmaker);
                    po_praybill.put("dmakedate", dmakedate);
                    responseJson.put("po_praybill", po_praybill);
                }else {
                    log.error("ReimbursementInfoAction execute error", "主表查询不到数据");
                    return WeaResult.fail("ReimbursementInfoAction execute error--主表查询不到数据");
                }
            }else {
                log.error("ReimbursementInfoAction execute error", "主表数据查询失败");
                return WeaResult.fail("ReimbursementInfoAction execute error--主表数据查询失败");
            }
            List<String> detailTableNameList = dataOperateUtil.getDetailTableName(wokflowId);
            String detailTableName = detailTableNameList.get(0);
            String detailTableSql = "select * from " + detailTableName + " where form_data_id = " + formDataId;
            log.error("ReimbursementInfoAction execute sql : " + detailTableSql);
            Map<String, Object> detailDatas = dataOperateUtil.executeForQuery(sourceType, groupId, detailTableSql);
            JSONArray po_praybill_b = new JSONArray();
            if(String.valueOf(detailDatas.get("status")).equals("OK")) {
                List<Map<String, Object>> detailRecords = (List<Map<String, Object>>) detailDatas.get("records");
                if (detailRecords.size() > 0) {
                    for (int i = 0; i < detailRecords.size(); i++) {
                        JSONObject detailJson = new JSONObject();
                        Map<String, Object> detailMap = detailRecords.get(i);
                        String pk_material = dataOperateUtil.null2String(detailMap.get("pk_material"));
                        String castunitid = dataOperateUtil.null2String(detailMap.get("castunitid"));;
                        String nastnum = dataOperateUtil.null2String(detailMap.get("nastnum"));
                        String vchangerate = dataOperateUtil.null2String(detailMap.get("vchangerate"));
                        String cunitid = dataOperateUtil.null2String(detailMap.get("cunitid"));
                        String nnum = dataOperateUtil.null2String(detailMap.get("nnum"));
                        String dreqdate = dataOperateUtil.null2String(detailMap.get("dreqdate"));
                        String dsuggestdate = dataOperateUtil.null2String(detailMap.get("dsuggestdate"));
                        String pk_employee = dataOperateUtil.getJobNum(dataOperateUtil.null2String(detailMap.get("pk_employee"))).toString();
                        String pk_suggestsupplier = dataOperateUtil.null2String(detailMap.get("pk_suggestsupplier"));
                        String pk_purchaseorg = dataOperateUtil.null2String(detailMap.get("pk_purchaseorg"));
                        detailJson.put("pk_material", pk_material);
                        detailJson.put("castunitid", castunitid);
                        detailJson.put("nastnum", nastnum);
                        detailJson.put("vchangerate", vchangerate);
                        detailJson.put("cunitid", cunitid);
                        detailJson.put("nnum", nnum);
                        detailJson.put("dreqdate", dreqdate);
                        detailJson.put("dsuggestdate", dsuggestdate);
                        detailJson.put("pk_employee", pk_employee);
                        detailJson.put("pk_suggestsupplier", pk_suggestsupplier);
                        detailJson.put("pk_purchaseorg", pk_purchaseorg);
                        po_praybill_b.add(detailJson);
                    }
                    responseJson.put("po_praybill_b", po_praybill_b);
                    String url = dataOperateUtil.getConfig("ip_address") + "/nccloud/api/nccloud/api/pu/praybillext/add";
                    JSONObject tokenInfo = thirdInfoUtil.getToken();
                    if (tokenInfo.get("success").equals("true")){
                        JSONObject data = tokenInfo.getJSONObject("data");
                        String token = data.getString("access_token");
                        JSONObject request = thirdInfoUtil.transferInfo(url, token, responseJson);
                        String success = request.get("success").toString();
                        if ("true".equals(success)){
                            return WeaResult.success(thirdInfoUtil.getResultMapForAction(weaverResultMap, "result", Action.SUCCESS, requestinfo.getRequestManager()));
                        }else {
                            log.info("PurchaseRequisitionInfoAction execute error--推送失败");
                            return WeaResult.fail("PurchaseRequisitionInfoAction execute error--推送失败");
                        }
                    }else {
                        log.info("PurchaseRequisitionInfoAction execute error--token调用失败");
                        return WeaResult.fail("PurchaseRequisitionInfoAction execute error--token调用失败");
                    }
                }else {
                    log.info("PurchaseRequisitionInfoAction execute error--查询不到数据");
                    return WeaResult.fail("PurchaseRequisitionInfoAction execute error--查询不到数据");
                }
            }else {
                log.info("PurchaseRequisitionInfoAction execute error--查询失败");
                return WeaResult.fail("PurchaseRequisitionInfoAction execute error--查询失败");
            }
        } catch (Exception e) {
            log.error("PurchaseRequisitionInfoAction execute error", e);
            return WeaResult.fail("PurchaseRequisitionInfoAction execute error", e);
        }
    }
}
