package com.weaver.secondev.action.info;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.enums.SourceType;
import com.weaver.esb.api.rpc.EsbServerlessRpcRemoteInterface;
import com.weaver.eteams.file.client.file.FileData;
import com.weaver.file.ud.api.FileDownloadService;
import com.weaver.secondev.util.DataOperateUtil;
import com.weaver.secondev.util.ThirdInfoUtil;
import com.weaver.verupgrade.file.ImageFileManager;
import com.weaver.verupgrade.interfaces.workflow.action.Action;
import com.weaver.verupgrade.soa.workflow.request.RequestInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: calyrex
 * @CreateTime: 2025-06-03
 * @Description: 应付单数据推送
 */

@Slf4j
@Service("workflow_action_ReimbursementInfoAction")
public class ReimbursementInfoAction implements EsbServerlessRpcRemoteInterface {

    @Autowired
    private RequestInfo requestInfoTemp;

    @Autowired
    private DataOperateUtil dataOperateUtil;

    @Autowired
    private ThirdInfoUtil thirdInfoUtil;

    @Autowired
    private FileDownloadService fileDownloadService;

    @Override
    public WeaResult<Map<String, Object>> execute(Map<String, Object> requestInfoMap) {
        log.error("ReimbursementInfoAction execute start");
        log.error("ReimbursementInfoAction execute params : " + requestInfoMap);
        try {
            Map<String, Object> weaverResultMap = new HashMap<>();
            Long requestIdTemp = Long.parseLong(String.valueOf(requestInfoMap.getOrDefault("requestid", -1)));
            Long userIdTemp = Long.parseLong(String.valueOf(requestInfoMap.getOrDefault("userid", -1)));
            RequestInfo requestinfo = requestInfoTemp.getRequestInfo(requestIdTemp, userIdTemp);
            String wokflowId = requestinfo.getWorkflowid();
            String tableName = requestinfo.getRequestManager().getBillTableName();
            String formDataId = dataOperateUtil.getFormDataId(String.valueOf(requestIdTemp));
            String sql = "select * from " + tableName + " where form_data_id = " + formDataId;
            log.error("ReimbursementInfoAction execute sql : " + sql);
            String sourceType = String.valueOf(SourceType.LOGIC);
            String groupId = "weaver-workflow-report-serviceworkflowreport";
            Map<String, Object> datas = dataOperateUtil.executeForQuery(sourceType, groupId, sql);
            JSONObject responseJson = new JSONObject();
            String xgfj  = "";
            if(String.valueOf(datas.get("status")).equals("OK")){
                List<Map<String,Object>> records = (List<Map<String,Object>>)datas.get("records");
                if (records.size()>0){
                    String pk_org_v = dataOperateUtil.null2String(records.get(0).get("pk_org_v"));
                    String pk_org = dataOperateUtil.null2String(records.get(0).get("pk_org"));
                    String billdate = dataOperateUtil.null2String(records.get(0).get("billdate"));
                    String busidate = dataOperateUtil.null2String(records.get(0).get("busidate"));
                    String objtype = "3";
                    String pk_tradetypeid = "D1";
                    String pk_busitype = "AP01";
                    String scomment = dataOperateUtil.null2String(records.get(0).get("scomment"));
                    String billmaker = dataOperateUtil.null2String(records.get(0).get("billmaker"));
                    String def21 = wokflowId;
                    String def22 = String.valueOf(requestIdTemp);
                    String pu_org = dataOperateUtil.getDepartmentCode(dataOperateUtil.null2String(records.get(0).get("pu_org"))).toString();
                    String pu_deptid_v = dataOperateUtil.getDepartmentCode(dataOperateUtil.null2String(records.get(0).get("pu_deptid_v"))).toString();
                    //报销人待确认
                    String pk_psndoc = dataOperateUtil.getJobNum(dataOperateUtil.null2String(records.get(0).get("pk_psndoc"))).toString();
                    String pk_currtype = "CNY";
                    String local_money = dataOperateUtil.null2String(records.get(0).get("local_money"));
                    String grouprate = dataOperateUtil.null2String(records.get(0).get("grouprate"));
                    String grouplocal = dataOperateUtil.null2String(records.get(0).get("grouplocal"));
                    String globalrate = dataOperateUtil.null2String(records.get(0).get("globalrate"));
                    String globallocal = dataOperateUtil.null2String(records.get(0).get("globallocal"));
                    String rate = dataOperateUtil.null2String(records.get(0).get("rate"));
                    String money = dataOperateUtil.null2String(records.get(0).get("money"));
                    xgfj  = dataOperateUtil.null2String(records.get(0).get("xgfj"));
                    responseJson.put("pk_org_v", pk_org_v);
                    responseJson.put("pk_org", pk_org);
                    responseJson.put("billdate", billdate);
                    responseJson.put("busidate", busidate);
                    responseJson.put("objtype", objtype);
                    responseJson.put("pk_tradetypeid", pk_tradetypeid);
                    responseJson.put("pk_busitype", pk_busitype);
                    responseJson.put("scomment", scomment);
                    responseJson.put("billmaker", billmaker);
                    responseJson.put("def21", def21);
                    responseJson.put("def22", def22);
                    responseJson.put("pu_org", pu_org);
                    responseJson.put("pu_deptid_v", pu_deptid_v);
                    responseJson.put("pk_psndoc", pk_psndoc);
                    responseJson.put("pk_currtype", pk_currtype);
                    responseJson.put("local_money", local_money);
                    responseJson.put("grouprate", grouprate);
                    responseJson.put("grouplocal", grouplocal);
                    responseJson.put("globalrate", globalrate);
                    responseJson.put("globallocal", globallocal);
                    responseJson.put("rate", rate);
                    responseJson.put("money", money);
                }else {
                    log.error("ReimbursementInfoAction execute error", "主表查询不到数据");
                    return WeaResult.fail("ReimbursementInfoAction execute error--主表查询不到数据");
                }
            }else {
                log.error("ReimbursementInfoAction execute error", "主表数据查询失败");
                return WeaResult.fail("ReimbursementInfoAction execute error--主表数据查询失败");
            }

            List<String> detailTableNameList = dataOperateUtil.getDetailTableName(wokflowId);
            String detailTableName = detailTableNameList.get(0);
            String detailTableSql = "select * from " + detailTableName + " where form_data_id = " + formDataId;
            log.error("ReimbursementInfoAction execute sql : " + detailTableSql);
            Map<String, Object> detailDatas = dataOperateUtil.executeForQuery(sourceType, groupId, detailTableSql);
            if(String.valueOf(detailDatas.get("status")).equals("OK")){
                List<Map<String,Object>> detailRecords = (List<Map<String,Object>>)detailDatas.get("records");
                if (detailRecords.size()>0){
                    JSONArray items = new JSONArray();
                    for (int i = 0; i < detailRecords.size(); i++) {
                        JSONObject detailJson = new JSONObject();
                        Map<String, Object> detailMap = detailRecords.get(i);
                        String project = dataOperateUtil.null2String(detailMap.get("project"));
                        String objtype = dataOperateUtil.null2String(detailMap.get("objtype"));
                        String pk_deptid_v = dataOperateUtil.getDepartmentCode(dataOperateUtil.null2String(detailMap.get("pk_deptid_v"))).toString();
                        String pk_psndoc = dataOperateUtil.getJobNum(dataOperateUtil.null2String(detailMap.get("pk_psndoc"))).toString();
                        String pk_subjcode = dataOperateUtil.null2String(detailMap.get("pk_subjcode"));
                        String pk_currtype = "CNY";
                        String rate = "1";
                        String def21 = dataOperateUtil.null2String(detailMap.get("def21"));
                        String def22 = dataOperateUtil.null2String(detailMap.get("def22"));
                        String money_cr = dataOperateUtil.null2String(detailMap.get("money_cr"));
                        String local_money_cr = dataOperateUtil.null2String(detailMap.get("local_money_cr"));
                        String axcodeid = dataOperateUtil.null2String(detailMap.get("axcodeid"));
                        String taxrate = dataOperateUtil.null2String(detailMap.get("taxrate"));
                        String local_tax_cr = dataOperateUtil.null2String(detailMap.get("local_tax_cr"));
                        String notax_cr = dataOperateUtil.null2String(detailMap.get("notax_cr"));
                        String local_notax_cr = dataOperateUtil.null2String(detailMap.get("local_notax_cr"));
                        String recaccount = dataOperateUtil.null2String(detailMap.get("recaccount"));
                        String grouprate = dataOperateUtil.null2String(detailMap.get("grouprate"));
                        String groupcrebit = dataOperateUtil.null2String(detailMap.get("groupcrebit"));
                        String globalrate = dataOperateUtil.null2String(detailMap.get("globalrate"));
                        String globalcrebit = dataOperateUtil.null2String(detailMap.get("globalcrebit"));
                        String quantity_cr = "1";
                        String price = dataOperateUtil.null2String(detailMap.get("price"));
                        String taxcodeid = dataOperateUtil.null2String(detailMap.get("taxcodeid"));
                        detailJson.put("project", project);
                        detailJson.put("objtype", objtype);
                        detailJson.put("pk_deptid_v", pk_deptid_v);
                        detailJson.put("pk_psndoc", pk_psndoc);
                        detailJson.put("pk_subjcode", pk_subjcode);
                        detailJson.put("pk_currtype", pk_currtype);
                        detailJson.put("rate", rate);
                        detailJson.put("def21", def21);
                        detailJson.put("def22", def22);
                        detailJson.put("money_cr", money_cr);
                        detailJson.put("local_money_cr", local_money_cr);
                        detailJson.put("axcodeid", axcodeid);
                        detailJson.put("taxrate", taxrate);
                        detailJson.put("local_tax_cr", local_tax_cr);
                        detailJson.put("notax_cr", notax_cr);
                        detailJson.put("local_notax_cr", local_notax_cr);
                        detailJson.put("recaccount", recaccount);
                        detailJson.put("grouprate", grouprate);
                        detailJson.put("groupcrebit", groupcrebit);
                        detailJson.put("globalrate", globalrate);
                        detailJson.put("globalcrebit", globalcrebit);
                        detailJson.put("quantity_cr", quantity_cr);
                        detailJson.put("price", price);
                        detailJson.put("taxcodeid", taxcodeid);
                        items.add(detailJson);
                    }
                    responseJson.put("items", items);
                    String url = dataOperateUtil.getConfig("ip_address") + "/nccloud/api/arap/payablebill/insert";
                    JSONObject tokenInfo = thirdInfoUtil.getToken();
                    if (tokenInfo.get("success").equals("true")){
                        JSONObject data = tokenInfo.getJSONObject("data");
                        String token = data.getString("access_token");
                        JSONObject request = thirdInfoUtil.transferInfo(url, token, responseJson);
                        String success = request.get("success").toString();
                        if ("true".equals(success)){
                            JSONObject data1 = request.getJSONObject("data");
                            String pk_bill = data1.getString("pk_bill");
                            List<String> xgfjList = Arrays.asList(xgfj.split(","));
                            for (String fileId : xgfjList) {
                                FileData fileData = fileDownloadService.downloadFileLazy(Long.valueOf(fileId));
                                InputStream inputStream = fileData.getInputStream();
                                String fileName = dataOperateUtil.getFileName(fileId);
                                Boolean isUpdate = thirdInfoUtil.updateFile(pk_bill, fileName, inputStream);
                                if (!isUpdate){
                                    log.info("updateFile error:上传附件失败");
                                    return WeaResult.fail("上传附件失败");
                                }
                            }
                        }else {
                            log.info("transferInfo error:{}", request);
                            return WeaResult.fail("推送表单数据失败");
                        }
                    }
                    return WeaResult.success(thirdInfoUtil.getResultMapForAction(weaverResultMap, "result", Action.SUCCESS, requestinfo.getRequestManager()));
                }else {
                    log.info("getToken error:明细表为空" );
                    return WeaResult.fail("明细表为空");
                }
            }else {
                log.info("getToken error:查询数据失败");
                return WeaResult.fail("查询数据失败");
            }
        } catch (Exception e) {
            log.error("ReimbursementInfoAction execute error", e);
            return WeaResult.fail("ReimbursementInfoAction execute error", e);
        }
    }
}
